==> darkradiant_2.11.0.orig.tar.gz: DarkRadiant-2.11.0/radiantcore/selection/clipboard/Clipboard.cpp <==
#include "Clipboard.h"

#include "i18n.h"
#include "iselection.h"
#include "igrid.h"
#include "icameraview.h"
#include "imapformat.h"
#include "iclipboard.h"

#include "map/Map.h"
#include "brush/FaceInstance.h"
#include "map/algorithm/Import.h"
#include "selection/algorithm/General.h"
#include "selection/algorithm/Transformation.h"
#include "command/ExecutionNotPossible.h"

namespace selection
{

namespace clipboard
{

void pasteToMap()
{
	if (!module::GlobalModuleRegistry().moduleExists(MODULE_CLIPBOARD))
	{
		throw cmd::ExecutionNotPossible(_("No clipboard module attached, cannot perform this action."));
	}

    std::stringstream stream(GlobalClipboard().getString());
	map::algorithm::importFromStream(stream);
}

void copy(const cmd::ArgumentList& args)
{
	if (FaceInstance::Selection().empty())
    {
		if (!module::GlobalModuleRegistry().moduleExists(MODULE_CLIPBOARD))
		{
			throw cmd::ExecutionNotPossible(_("No clipboard module attached, cannot perform this action."));
		}

		// When exporting to the system clipboard, use the portable format
		auto format = GlobalMapFormatManager().getMapFormatByName(map::PORTABLE_MAP_FORMAT_NAME);

        // Stream selected objects into a stringstream
        std::stringstream out;
        GlobalMap().exportSelected(out, format);

        // Copy the resulting string to the clipboard
		GlobalClipboard().setString(out.str());
	}
	else
	{
		algorithm::pickShaderFromSelection(args);
	}
}

void paste(const cmd::ArgumentList& args)
{
	if (FaceInstance::Selection().empty())
    {
		UndoableCommand undo("paste");
		pasteToMap();
	}
	else
	{
		algorithm::pasteShaderToSelection(args);
	}
}

void pasteToCamera(const cmd::ArgumentList& args)
{
	try
	{
		auto& camWnd = GlobalCameraManager().getActiveView();

		UndoableCommand undo("pasteToCamera");
		pasteToMap();

		// Work out the delta
		Vector3 mid = algorithm::getCurrentSelectionCenter();
		Vector3 delta = camWnd.getCameraOrigin().getSnapped(GlobalGrid().getGridSize()) - mid;

		// Move to camera
		algorithm::translateSelected(delta);
	}
	catch (const std::runtime_error& ex)
	{
		rError() << "Cannot paste to camera: " << ex.what() << std::endl;
	}
}

} // namespace

} // namespace

==> darkradiant_2.14.0.orig.tar.gz: DarkRadiant-2.14.0/radiantcore/selection/clipboard/Clipboard.cpp <==
#include "Clipboard.h"

#include "i18n.h"
#include "iselection.h"
#include "igrid.h"
#include "icameraview.h"
#include "imapformat.h"
#include "iclipboard.h"
#include "ishaderclipboard.h"
#include "string/trim.h"

#include "map/Map.h"
#include "brush/FaceInstance.h"
#include "map/algorithm/Import.h"
#include "selection/algorithm/General.h"
#include "selection/algorithm/Transformation.h"
#include "command/ExecutionNotPossible.h"
#include "command/ExecutionFailure.h"

namespace selection
{

namespace clipboard
{

void pasteToMap()
{
	if (!module::GlobalModuleRegistry().moduleExists(MODULE_CLIPBOARD))
	{
		throw cmd::ExecutionNotPossible(_("No clipboard module attached, cannot perform this action."));
	}

    std::stringstream stream(GlobalClipboard().getString());
	map::algorithm::importFromStream(stream);
}

void copy(const cmd::ArgumentList& args)
{
	if (FaceInstance::Selection().empty())
    {
		if (!module::GlobalModuleRegistry().moduleExists(MODULE_CLIPBOARD))
		{
			throw cmd::ExecutionNotPossible(_("No clipboard module attached, cannot perform this action."));
		}

		// When exporting to the system clipboard, use the portable format
		auto format = GlobalMapFormatManager().getMapFormatByName(map::PORTABLE_MAP_FORMAT_NAME);

        // Stream selected objects into a stringstream
        std::stringstream out;
        GlobalMap().exportSelected(out, format);

        // Copy the resulting string to the clipboard
		GlobalClipboard().setString(out.str());
	}
	else
	{
		algorithm::pickShaderFromSelection(args);
	}
}

std::string getMaterialNameFromClipboard()
{
    if (!module::GlobalModuleRegistry().moduleExists(MODULE_CLIPBOARD))
    {
        return std::string();
    }

    auto candidate = GlobalClipboard().getString();
    string::trim(candidate);

    // If we get a single line, check if the contents point to a material we know
    if (!candidate.empty() && candidate.find('\n') == std::string::npos &&
        GlobalMaterialManager().materialExists(candidate))
    {
        return candidate;
    }
    
    return std::string();
}

void paste(const cmd::ArgumentList& args)
{
	if (FaceInstance::Selection().empty())
    {
        auto clipboardMaterial = getMaterialNameFromClipboard();

        if (!clipboardMaterial.empty())
        {
            UndoableCommand undo("pasteMaterialFromClipboard");

            // Activate the material name in the shader clipboard, but don't overwrite
            // anything there if the material is already matching to not overwrite Face/Patch information
            if (GlobalShaderClipboard().getShaderName() != clipboardMaterial)
            {
                GlobalShaderClipboard().setSourceShader(clipboardMaterial);
            }

            algorithm::pasteShaderToSelection(args);
            return;
        }

        // Try to parse the map and apply it
        UndoableCommand undo("paste");
        pasteToMap();
	}
	else
	{
		algorithm::pasteShaderToSelection(args);
	}
}

void pasteToCamera(const cmd::ArgumentList& args)
{
	try
	{
		auto& camWnd = GlobalCameraManager().getActiveView();

		UndoableCommand undo("pasteToCamera");
		pasteToMap();

		// Work out the delta
		Vector3 mid = algorithm::getCurrentSelectionCenter();
		Vector3 delta = camWnd.getCameraOrigin().getSnapped(GlobalGrid().getGridSize()) - mid;

		// Move to camera
		algorithm::translateSelected(delta);
	}
	catch (const std::runtime_error& ex)
	{
		rError() << "Cannot paste to camera: " << ex.what() << std::endl;
	}
}

} // namespace

} // namespace
